using Server;
using Server.Mobiles;
using System;
using System.Collections;

namespace Server.Custom
{
	public enum Class
	{
		None = -1,
		
		Naturalist = 0,
		Warrior,
		Paladin,
		Assassin,
		Rogue,
		Monk,
		Bard,
		Illusionist,
		Ranger,
		Tinker,
		Merchant,
		Cleric,
		Magician,
		Scholar
	}

	[Flags]
	public enum RacesAllowedTo		// Human		Elf		Dwarf	HalfElf	Hobbit	HalfOrc	Gnome
	{
		BeNone			=	0x0000,	// Yes		Yes		Yes		Yes		Yes		Yes		Yes
		BeNaturalist	=	0x0001,	// Yes		Yes		Yes		Yes		Yes		Yes		Yes
		BeWarrior		=	0x0002, // Yes		Yes		Yes		Yes		Yes		Yes		Yes
		BePaladin		=	0x0004,	// Yes		Yes		Yes		Yes		Yes		Yes		Yes
		BeAssassin		=	0x0008,	// Yes		Yes		Yes		Yes		Yes		Yes		Yes
		BeRogue			=	0x0010, // Yes		Yes		Yes		Yes		Yes		Yes		Yes
		BeMonk			=	0x0020, // Yes		Yes		Yes		Yes		Yes		Yes		Yes
		BeBard			=	0x0040, // Yes		Yes		Yes		Yes		Yes		Yes		Yes
		BeIllusionist	=	0x0080, // Yes		Yes		Yes		Yes		Yes		Yes		Yes
		BeRanger		=	0x0100,	// Yes		Yes		Yes		Yes		Yes		Yes		Yes
		BeTinker		=	0x0200,	// Yes		Yes		Yes		Yes		Yes		Yes		Yes
		BeMerchant		=	0x0400,	// Yes		Yes		Yes		Yes		Yes		Yes		Yes
		BeCleric		=	0x0800,	// Yes		Yes		Yes		Yes		Yes		Yes		Yes
		BeMagician		=	0x1000,	// Yes		Yes		Yes		Yes		Yes		Yes		Yes
		BeScholar		=	0x2000,	// Yes		Yes		Yes		Yes		Yes		Yes		Yes
	}								// 0x3FFF		0x3FFF	0x3FFF	0x3FFF	0x3FFF	0x3FFF	0x3FFF

	public class ClassEntry
	{
		private Class m_Class;
		private string[] m_Name;
		private string m_Desc;
		private SkillEntry m_Skills;
		private int[] m_Points;

		public Class Class { get{ return m_Class; } }
		public string[] Rank { get{ return m_Name; } 
			set { m_Name = value; } 
		}
		public string Description { get{ return m_Desc; } }
		public SkillEntry ClassSkills { get{ return m_Skills; } }
		public int[] LevelPoints { get{ return m_Points; } }
		 
		private NewMobile m_Owner;
		public NewMobile Owner{get{ return m_Owner; }set{ m_Owner = value; }}

		public ClassEntry( Class clas, string[] name, string description, 
			SkillEntry classSkills, int[] levelPoints )
		{		
			m_Class = clas;
			int lenName = name.Length;
			m_Name = new string[lenName];
			for (int x=0; x<lenName; x++)
			{
				m_Name[x] = name[x];
			}
			m_Desc = description;
			m_Skills = new SkillEntry(classSkills.Class, classSkills.SkillCount, classSkills.ClassSkills, classSkills.NewSkillCount, classSkills.NewClassSkills);
			int lenPoints = levelPoints.Length;
			m_Points = new int[lenPoints];
			for (int x=0; x<lenPoints; x++)
			{
				m_Points[x] = levelPoints[x];
			}
		}

		public ClassEntry( NewMobile owner )
		{
			m_Owner = owner;
		}

		private static Hashtable m_Table = new Hashtable();
		public static Hashtable Table{ get{ return m_Table; } }
		
		public static ClassEntry GetEntry( Class clas )
		{
			return ( m_Table[clas] as ClassEntry );
		}
		
		public static void Register( ClassEntry classEntry )
		{
			if ( !m_Table.ContainsKey( classEntry.Class ) )
				m_Table[classEntry.Class] = classEntry;
			else
				throw new Exception( String.Format( "Class {0} exists.", classEntry.Class.ToString() ) );
		}
	}

	public sealed class NoneClass
	{
		public static void Initialize()
		{
			SkillNameValue[] snv = new SkillNameValue[]{
					new SkillNameValue(SkillName.Anatomy, 0) };

			NewSkillNameValue[] newSnv = new NewSkillNameValue[]{
					new NewSkillNameValue(NewSkillName.Gardening, 0),
					new NewSkillNameValue(NewSkillName.Forestry, 0),
					new NewSkillNameValue(NewSkillName.Taxidermy, 0),
					new NewSkillNameValue(NewSkillName.Boating, 0),
					new NewSkillNameValue(NewSkillName.Sailing, 0)
			};
			int[] points = new int[]{ 0, 0 };
			Class clas = Class.None;
			string[] name = new string[]{ "None", "None" };
			string description = "No Class Chosen.";
			SkillEntry classSkills = new SkillEntry( clas, 1, snv, 5, newSnv );
			ClassEntry.Register( new ClassEntry( clas, name, description, classSkills, points ) );
		}
	}
}
