using System;
using System.Collections;
using Server;
using Server.Gumps;
using Server.Mobiles;
using Server.Network;

namespace Server.Custom
{
	public class ChooseClassGump : Gump
	{
		private static int m_Choice;
		private NewMobile m_Mobile;
		private static Class m_OldClass;
		private static int m_Exp;
		private static int m_Level;

		public ChooseClassGump( NewMobile mobile, bool repeat ) : base( 50, 50 )
		{
			m_Mobile = mobile;
			if ( !repeat )
			{
				m_OldClass = m_Mobile._Class;
				m_Exp = m_Mobile._Experience;
				m_Level = m_Mobile._ClassLevel;
			}
			mobile.CloseGump( typeof( ChooseClassGump ) );

			AddPage( 0 );
			AddBackground( 0, 0, 476, 540, 0x13BE );
			AddImageTiled( 150, 30, 316, 205, 0xA40 );
			AddAlphaRegion( 150, 30, 316, 205 );

			AddLabel( 10, 7, 2100, "Choose Class" );

			string description = ClassEntry.GetEntry( m_Mobile._Class ).Description;

			AddHtml( 160, 40, 296, 185, description, true, true );
			AddImageTiled( 10, 30, 101, 460, 0xA40 );
			AddAlphaRegion( 10, 30, 101, 460 );

			if ( m_Mobile.NewLevel.CanGainLevel(m_Mobile) )
			{
				AddLabel( 370, 240, 2100, "Level Up" );
				AddButton( 436, 240, 0xFB7, 0xFB9, 1, GumpButtonType.Reply, 0 );
			}
			else 
			{
				AddLabel( 320, 240, 2100, "Choose This Class" );
				AddButton( 436, 240, 0xFB7, 0xFB9, 2, GumpButtonType.Reply, 0 );
			}
			
			int count = 0;
				
			foreach( ClassEntry Class in ClassEntry.Table.Values )
			{
				int y = 30 * (++count);
				AddButton( 115, y, 0xFAE, 0xFB0, (int)Class.Class + 4, GumpButtonType.Reply, 0 );
				AddLabel( 15, y, Class.Class == m_Mobile._Class ? 2124 : 2100, Class.Class.ToString() );
			}

			if ( m_Mobile._Class != Server.Custom.Class.None )
			{
				for ( int x=0; x<5; x++ )
				{
					AddLabel( 160, 275 + (x * 25), 2100, m_Mobile.NewSkills[x].Name.ToString() );
					AddLabel( 360, 275 + (x * 25), 2100, m_Mobile.NewSkills[x].Value.ToString() );
				}
			}
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			m_Choice = info.ButtonID;
			char[] vowels = new Char[]{'A', 'E', 'I', 'O', 'U', 'a', 'e', 'i', 'o', 'u'};
			Class c = m_Mobile._Class;

			if ( m_Choice == 1 )
			{
				m_Mobile.NewLevel.GainLevel();
			}
			else if ( m_Choice == 2 )
			{
				if ( c == Class.None )
				{
					m_Mobile.SendMessage("You have chosen to have no class. Please try again.");
					m_Mobile.SendGump( new ChooseClassGump( m_Mobile, true ) );
				}
				else
				{
					m_Mobile.SendMessage("You are now {1} {0}!", c.ToString(),
					c.ToString().LastIndexOfAny( vowels, 0, 1 ) > 0 ? "an" : "a" );
				}
				m_Mobile.CloseGump( typeof( ChooseClassGump ) );
			}
			else if ( m_Choice >= 4 )
			{
				if ( m_Mobile._Experience > 0 )
					m_Mobile.SendGump( new Gumps.SureGump( "Abandon Your Class?", 1400,
						"Are you sure you want to abandon your current class? You will lose all of the Experience you have gained, and your Class attributes.",
						1400, 300, 300, new SureGumpCallback( ChangeClass_Callback ) ) );
				else
				{
					ChangeClass( m_Mobile );
					m_Mobile.SendGump( new ChooseClassGump( m_Mobile, true ) );
				}
			}
			else
			{
				m_Mobile._Class = m_OldClass;
				m_Mobile.NewLevel.Candidate = m_Mobile;
				m_Mobile.NewLevel.CurrentLevel = m_Level;
				m_Mobile._Experience = m_Exp;
				m_Mobile.NewLevel.ClassEntry = ClassEntry.GetEntry( m_Mobile._Class );
				m_Mobile.SendMessage("All Changes Canceled.");
			}
		}

		public static void ChangeClass_Callback( NewMobile from, bool okay )
		{
			if ( okay )
			{
				ChangeClass( from );
				from.SendGump( new ChooseClassGump( from, true ) );
			}
			else
			{
				from.SendGump( new ChooseClassGump( from, true ) );
			}
		}

		private static void ChangeClass( NewMobile from )
		{
			from._Class = (Class)( m_Choice - 4 );
			from.NewLevel.Candidate = from;
			from.NewLevel.CurrentLevel = 0;
			from._Experience = 0;
			from.NewLevel.ClassEntry = ClassEntry.GetEntry( from._Class );
			for (int x=0; x<5; x++)
				from.NewSkills[x] = from.NewLevel.ClassEntry.ClassSkills.NewClassSkills[x];
		}
	}
}