using System;
using System.Collections;
using Server;
using Server.Gumps;
using Server.Mobiles;
using Server.Network;
using Server.Commands;

namespace Server.Custom
{
	public class NewSkillsCommand
	{
		public static void Initialize()
		{
			CommandSystem.Register( "NewSkills", AccessLevel.Player, new CommandEventHandler( NewSkills_OnCommand ) );
		}
		
		[Usage( "NewSkills" )]
		[Description( "Shows a Player's New Skills." )]
		private static void NewSkills_OnCommand( CommandEventArgs e )
		{
			if (e.Mobile is NewMobile)
			{
				NewMobile m = e.Mobile as NewMobile;
				m.SendGump( new NewSkillsGump( m ) );
			}
			else e.Mobile.SendMessage("You may not use this command!");
		}
	}

	public class NewSkillsGump : Gump
	{
		private static NewMobile m_Mobile;
		private string[] labels = new string[]{"Player Race:","Player Class:","Current Level / Rank:","XP Have / XP Needed:","Strength:","Intelligence:","Dexterity:","Constitution:","Charisma:","Wisdom:"};
		private string[] values = new string[]{"","","","","","","","","",""};
		private string[] zlabels = new string[]{"","","","",""};
		private string[] zvalues = new string[]{"","","","",""};

		public NewSkillsGump( NewMobile mobile ) : base( 0, 0 )
		{
			m_Mobile = mobile;
			m_Mobile.CloseGump( typeof( NewSkillsGump ) );
			try
			{
				m_Mobile.CloseGump( typeof( NewStatsGump ) );
			}
			catch
			{
				Console.WriteLine("Unable to close Stats Gump.");
			}
			SetValues( m_Mobile );

			AddPage( 0 );
			AddBackground( 670, 300, 120, 20, 9200 );

			AddLabel( 671, 301, 2222, "STATS" );
			AddButton( 716, 303, 1210, 1210, 22, GumpButtonType.Page, 2 );
			AddLabel( 731, 301, 2222, "SKILLS" );
			AddButton( 776, 303, 1210, 1210, 21, GumpButtonType.Page, 1 );

			AddPage( 1 );

			AddAlphaRegion( 670, 320, 200, 250 );
			for ( int x=0; x<5; x++ )
			{
				AddLabel( 687, 330 + ( x * 20 ), 2222, zlabels[x] );
				AddLabel( 850, 330 + ( x * 20 ), 2222, zvalues[x] );
				
				try
				{
					if ( NewSkillInfo.Table[(int)m_Mobile.NewSkills[x].Name].ClickToUse )
						AddButton( 670, 333 + ( x * 20 ), 1210, 1210, x+10, GumpButtonType.Reply, 0 );
				}
				catch
				{
					m_Mobile.SendMessage(string.Format("Exception disarmed, check console for more info. ( x = {0} )", x ));
				}
			}
			AddLabel( 740, 455, 2200, "Refresh" );
			AddButton( 800, 457, 1210, 1210, 1, GumpButtonType.Reply, 0 );

			AddPage( 2 );

			AddAlphaRegion( 670, 320, 200, 250 );
			for ( int x=0; x<10; x++ )
			{
				AddLabel( 680, 330 + ( x * 20 ), 2222, labels[x] );
				AddLabel( 850, 330 + ( x * 20 ), 2222, values[x] );
			}
			AddLabel( 740, 545, 2200, "Refresh" );
			AddButton( 800, 547, 1210, 1210, 2, GumpButtonType.Reply, 0 );
		}
	
		public override void OnResponse( NetState state, RelayInfo info )
		{
			NewMobile m = state.Mobile as NewMobile;
			switch(info.ButtonID)
			{
				case 1: m.SendGump( new NewSkillsGump( m ) ); break;
				case 2: m.SendGump( new NewStatsGump( m ) ); break;
				case 10: m.UseNewSkill( m.NewSkills[0].Name ); break;
				case 11: m.UseNewSkill( m.NewSkills[1].Name ); break;
				case 12: m.UseNewSkill( m.NewSkills[2].Name ); break;
				case 13: m.UseNewSkill( m.NewSkills[3].Name ); break;
				case 14: m.UseNewSkill( m.NewSkills[4].Name ); break;
				default: m.CloseGump( typeof( NewSkillsGump ) ); break;
			}
		}

		private void SetValues( NewMobile m )
		{
			values[0] = m._Race.ToString();
			values[1] = m._Class.ToString();
			values[2] = m._ClassLevel.ToString() + " / " + m._ClassRank;
			values[3] = m._Experience.ToString() + " / " + m._NextLevelPointsNeeded.ToString();
			values[4] = m.ActualStrength.ToString();
			values[5] = m.ActualIntelligence.ToString();
			values[6] = m.ActualDexterity.ToString();
			values[7] = m.ActualConstitution.ToString();
			values[8] = m.ActualCharisma.ToString();
			values[9] = m.ActualWisdom.ToString();
			for (int x = 0; x<5; x++)
			{
				zvalues[x] = m.NewSkills[x].Value.ToString();
				zlabels[x] = m.NewSkills[x].Name.ToString();
			}
		}
	}
}