using System;
using Server;
using Server.Mobiles;
using Server.Gumps;
using Server.Network;

namespace Server.Custom
{
	public class RaiseSkillsGump : Gump
	{
		private static int m_Points;
		private NewMobile m_Mobile;
		private static int[] m_StartValue = new int[]{0,0,0,0,0};
		private static NewSkillNameValue nsnv;

		public RaiseSkillsGump( NewMobile mobile, int pointsleft, int[] values ) : this( mobile, pointsleft, false )
		{
			for (int x=0; x<5; x++)
				m_StartValue[x] = values[x];
		}

		public RaiseSkillsGump( NewMobile mobile, int pointsleft, bool first ) : base( 50, 50 )
		{
			m_Points = pointsleft;
			m_Mobile = mobile;

			if ( first )
			{
				for (int x=0; x<5; x++)
					m_StartValue[x] = m_Mobile.NewSkills[x].Value;
			}

			m_Mobile.CloseGump( typeof(RaiseSkillsGump) );

			AddPage( 0 );

			AddBackground( 0, 0, 476, 280, 0x13BE );

			AddLabel( 10, 7, 2100, "Choose Skills" );

			AddImageTiled( 10, 30, 101, 230, 0xA40 );
			AddAlphaRegion( 10, 30, 101, 230 );

			AddLabel( 10, 260, 2100, "Points Left: " + m_Points.ToString() );

			if ( m_Points == 0 )
			{
				AddButton( 205, 255, 0xFB7, 0xFB9, 1, GumpButtonType.Reply, 0 ); // OK button
			}

			AddImage( 170, -10, 0x58A );

			for ( int i = 0; i < 5; i++ )
			{
				int cost = i + 1;
				int y = 30 * ( i + 1 );

				nsnv = m_Mobile.NewSkills[i];

				AddLabel( 10, y, 2124, nsnv.Name.ToString() );

				AddLabel( 90, y, 2100, nsnv.Value.ToString() );

				AddLabel( 160, y, 2220, String.Format( "(Cost: {0} points)", cost ) );

				if ( CanLowerSkill( nsnv, i ) )
					AddButton( 120, y, 0x1519, 0x1519, 1000 + i, GumpButtonType.Reply, 0 ); // Decrease

				if ( CanRaiseSkill( nsnv, cost ) )
					AddButton( 140, y, 0x151A, 0x151A, 2000 + i, GumpButtonType.Reply, 0 ); // Increase
			}
		}

		public bool CanLowerSkill( NewSkillNameValue skill, int pos )
		{
			if ( skill.Value > m_StartValue[pos] )
				return true;
			else return false;
		}

		public bool CanRaiseSkill( NewSkillNameValue skill, int price )
		{
			if (( m_Points >= price ) && ( skill.Value < 100 ))
				return true;
			else return false;
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			NewMobile from = state.Mobile as NewMobile;

			if ( info.ButtonID == 1 ) // "Ok"
			{
				from.CloseGump( typeof( RaiseSkillsGump ) );
			}
			else if ( info.ButtonID >= 2000 ) // Increase
			{
				++from.NewSkills[info.ButtonID - 2000].Value;
				m_Points -= ((info.ButtonID - 2000) + 1);
				from.SendGump( new RaiseSkillsGump( from, m_Points, m_StartValue ) );
			}
			else if ( info.ButtonID >= 1000 ) // Decrease
			{
				--from.NewSkills[info.ButtonID - 1000].Value;
				m_Points += ((info.ButtonID - 1000) + 1);
				from.SendGump( new RaiseSkillsGump( from, m_Points, m_StartValue ) );
			}
			else
			{
				from.CloseGump( typeof( RaiseSkillsGump ) );
			}
		}
	}
}