using System;
using Server;
using Server.Mobiles;
using Server.Gumps;
using Server.Network;

namespace Server.Custom
{
	public class RaiseStatsGump : Gump
	{
		private int m_Points;
		private NewMobile m_Mobile;

		public RaiseStatsGump( NewMobile mobile ) : this( mobile, 20 )
		{
		}

		public RaiseStatsGump( NewMobile mobile, int points ) : base( 50, 50 )
		{
			mobile.CloseGump( typeof( RaiseStatsGump ) );

			m_Mobile = mobile;
			m_Points = points;
			
			AddPage( 0 );

			AddBackground( 0, 0, 240, 280, 0x13BE );
			AddLabel( 10, 7, 2100, "Raise Stats" );
			AddImageTiled( 10, 30, 186, 220, 0xA40 );
			AddLabel( 10, 260, 2100, "Points Left: " + m_Points.ToString() );
			
			if ( m_Points == 0 )
			{
				AddButton( 205, 255, 0xFB7, 0xFB9, 1000, GumpButtonType.Reply, 0 ); // OK button
			}
			
			AddEntry( Stat.Strength );
			AddEntry( Stat.Dexterity );
			AddEntry( Stat.Intelligence );
			AddEntry( Stat.Constitution );
			AddEntry( Stat.Charisma );
			AddEntry( Stat.Wisdom );
		}
		
		private int m_EntryCount;
		
		public void AddEntry( Stat stat )
		{						
			int y = 30 * ++m_EntryCount;
		
			AddLabel( 10, y, 2124, stat.ToString() );
			
			AddLabel( 90, y, 2100, m_Mobile.GetActual( stat ).ToString() );
			
			AddLabel( 135, y, 2220, String.Format( "({0} points)", GetRequiredPoints( stat ) ) );
			
			if ( CanDecreaseStat( stat ) )
				AddButton( 200, y, 0x1519, 0x1519, (int)stat + 1, GumpButtonType.Reply, 0 );

			if ( CanIncreaseStat( stat ) )
				AddButton( 220, y, 0x151A, 0x151A, (int)stat + 101, GumpButtonType.Reply, 0 );
		}
		
		public int GetLowerBound( Stat stat )
		{
			RaceEntry race = RaceEntry.GetEntry( m_Mobile._Race );
			
			return ( 6 + race.RaceMods[ stat ] );
		}
		
		public int GetUpperBound( Stat stat )
		{
			return ( GetLowerBound( stat ) + 10 );
		}
		
		public int GetRequiredPoints( Stat stat )
		{
			return GetRequiredPoints( stat, 0 );
		}
		
		public int GetRequiredPoints( Stat stat, int scoreOffset )
		{
			int standardScore = m_Mobile.GetActual( stat ) + ( 6 - GetLowerBound( stat ) ) + scoreOffset;
			
			if ( standardScore >= 17 )
				return 3;
			else if ( standardScore >= 15 )
				return 2;
			else
				return 1;
		}
		
		private bool CanIncreaseStat( Stat stat )
		{
			if ( GetRequiredPoints( stat ) > m_Points )
				return false;
			else if ( m_Mobile.GetActual( stat ) >= GetUpperBound( stat ) )
				return false;

			return true;
		}
		
		private bool CanDecreaseStat( Stat stat )
		{
			return ( m_Mobile.GetActual( stat ) > GetLowerBound( stat ) );
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			if ( info.ButtonID == 1000 ) // OK Button
			{
				m_Mobile.CloseGump( typeof( RaiseStatsGump ) );
			}
			else if ( info.ButtonID >= 1 )
			{
				bool increase = false;
			
				int id = info.ButtonID - 1;
				if ( id >= 100 )
				{
					id -= 100;
					increase = true;
				}
				
				Stat stat = (Stat)id;
				
				if ( increase && CanIncreaseStat( stat ) )
				{
					m_Points -= GetRequiredPoints( stat );
					m_Mobile.NewStats[ stat ]++;
					
				}
				else if ( !increase && CanDecreaseStat( stat ) )
				{
					m_Points += GetRequiredPoints( stat, -1 );
					m_Mobile.NewStats[ stat ]--;
				}
				else
				{
					return;
				}
				
				m_Mobile.SendGump( new RaiseStatsGump( m_Mobile, m_Points ) );
			}
		}
	}
}