using System;
using Server.Gumps;
using Server.Items;
using Server.Mobiles;
using Server.Network;

namespace Server.Custom
{
	public class NewStatBall : Item
	{
		private int m_Points;
		
		[CommandProperty( AccessLevel.GameMaster )]
		public int Points { get { return m_Points; } set { m_Points = value; } }
		
		[Constructable]
		public NewStatBall() : this( 1, 5 )
		{
		}
		
		[Constructable]
		public NewStatBall( int pointsMin, int pointsMax ) : this( Utility.Random( pointsMin, pointsMax - pointsMin ) )
		{
		}
		
		[Constructable]
		public NewStatBall( int points ) : base( 0x1726 )
		{
			Movable = true;
			Hue = 0x489;
			Name = "a new skill ball";
			m_Points = points;
		}

		public NewStatBall( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
			
			writer.Write( (int) m_Points );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
			
			m_Points = reader.ReadInt();
		}

		public override void OnDoubleClick( Mobile m )
		{
			if ( !( m is NewMobile ) )
			{
				m.SendMessage( "You may not use this!");
				return;
			}
			
			NewMobile from = m as NewMobile;
			if ( from._Race == Race.None )
			{
				from.SendMessage( "You must choose a Race and a Class before you can raise stats!" );
			}
			else if ( from._Class == Class.None )
			{
				from.SendMessage( "You must choose a Class before you can raise stats!" );
			}
			else
			{
				from.SendGump( new RaiseStatsGump( from, m_Points ) );
				this.Consume();
			}
		}
	}
}