using System;
using System.Collections;
using Server;
using Server.Mobiles;

namespace Server.Custom
{
	public class LevelEntry
	{
		private ClassEntry m_ClassEntry;
		private Class m_Class;
		public ClassEntry ClassEntry { get { return m_ClassEntry; } set { m_ClassEntry = value; } }
		public SkillEntry SkillLevel { get { return ClassEntry.ClassSkills; } }
		public int SkillCount { get { return SkillLevel.SkillCount; } }
		public int NewSkillCount { get { return SkillLevel.NewSkillCount; } }
		private NewMobile m_Candidate;
		public NewMobile Candidate{ get { return m_Candidate; } set { m_Candidate = value; } }

		private int m_CurrentLevel;
		public int CurrentLevel{ get { return m_CurrentLevel; } set { m_CurrentLevel = value; } }

		public int Experience{ get { return m_Candidate._Experience; } 
			set { m_Candidate._Experience = value; } }
	
		public LevelEntry( NewMobile candidate )
		{
			m_CurrentLevel = 0;
			m_Candidate = candidate;
			m_ClassEntry = Server.Custom.ClassEntry.GetEntry( m_Candidate._Class );
			try
			{
				for (int x=0; x<5; x++)
				{
					m_Candidate.NewSkills[x].Name = ClassEntry.GetEntry(m_Candidate._Class).ClassSkills.NewClassSkills[x].Name;
					m_Candidate.NewSkills[x].Value = ClassEntry.GetEntry(m_Candidate._Class).ClassSkills.NewClassSkills[x].Value;
				}
				Console.WriteLine("Loading {0}'s saved Skills.", m_Candidate.Name);
			}
			catch
			{
				Console.WriteLine("Unable to load {0}'s saved Skills.", m_Candidate.Name);
			}
		}

		public string GetRank()
		{
			return ClassEntry.GetEntry( m_Candidate._Class ).Rank[m_CurrentLevel];
		}

		public void SetRank( string rank )
		{
			string[] ranks = new string[m_ClassEntry.Rank.Length];
			for (int x=0; x<m_ClassEntry.Rank.Length; x++)
			{
				if (x == CurrentLevel)
					ranks[x] = rank;
				else
					ranks[x] = m_ClassEntry.Rank[x];
			}
			m_ClassEntry.Rank = ranks;
		}

		public SkillNameValue[] GetSkills()
		{
			if (SkillCount < 1)
				return null;
			SkillNameValue[] snv = new SkillNameValue[SkillCount];
			for (int x=0; x<SkillCount; x++)
			{
				snv[x] = SkillLevel.ClassSkills[x];
			}
			return snv;
		}

		public NewSkillNameValue[] GetNewSkills()
		{
			if (NewSkillCount < 1)
				return null;
			NewSkillNameValue[] nsnv = new NewSkillNameValue[NewSkillCount];
			for (int x=0; x<NewSkillCount; x++)
			{
				nsnv[x] = SkillLevel.NewClassSkills[x];
			}
			return nsnv;
		}

		public int PointsNeeded()
		{
			if ( m_Candidate._Class == Class.None )
				return 1;
			else
			{
				try
				{
					if ( m_CurrentLevel < ClassEntry.GetEntry( m_Candidate._Class).Rank.Length )
						return (ClassEntry.GetEntry( m_Candidate._Class ).LevelPoints[m_CurrentLevel + 1] - Experience);
					else
						return 999999;
				}
				catch
				{
					return 999999;
				}
			}
		}
		
		public void GainLevel()
		{
			if ( CanGainLevel(m_Candidate) )
			{
				m_CurrentLevel++;
				m_Candidate.SendMessage("You gained a level! You are now level {0}.", m_CurrentLevel);
				m_Candidate.SendGump( new RaiseSkillsGump( m_Candidate, (25 + (5 * m_CurrentLevel)), true ) );
			}
			else
				m_Candidate.SendMessage("You lack the necessary Experience to advance.");
		}
		
		public bool CanGainLevel(NewMobile m)
		{
			try
			{
				if ( ( PointsNeeded() < 1 ) && ( m_CurrentLevel < ClassEntry.GetEntry( m._Class).Rank.Length ) )
					return true;
			}
			catch
			{
			}
			return false;
		}
		
		public void Serialize( NewMobile from, GenericWriter writer )
		{
			writer.Write( (int) 0 ); // version
			
			writer.Write( (int) m_CurrentLevel );
		}
		
		public void Deserialize( NewMobile from, GenericReader reader )
		{
			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_Candidate = from;
					m_ClassEntry = ClassEntry.GetEntry( m_Candidate._Class );
					m_CurrentLevel = reader.ReadInt();
					break;
				}
			}
		}
	}
}