using System;
using System.Collections;
using System.Collections.Generic;
using Server;
using Server.ContextMenus;
using Server.Mobiles;
using Server.Items;
using Server.Gumps;
using Server.Custom;

namespace Server.Custom
{
	public class NewMobile : PlayerMobile
	{
		private LevelEntry m_Level;
		/// <summary>
		/// NewLevel is the LevelEntry for the NewMobile class.
		/// It keeps track of the Rank, Experience until next Level, etc.
		/// </summary>
		public LevelEntry NewLevel { get { return m_Level; } set { m_Level = value; } }

		private Stats m_Stats;
		/// <summary>
		/// NewStats is the Stats for the NewMobile class.
		/// It keeps track of Race, Class, Stats, Bonuses, Caps, etc.
		/// </summary>
		public Stats NewStats { get { return m_Stats; } set { m_Stats = value; } }

		private NewSkillNameValue[] m_Skills;
		/// <summary>
		/// These are the NewSkillNameValue's for the NewMobile class.
		/// They include ClassSkill_1 through ClassSkill_5.
		/// </summary>
		public NewSkillNameValue[] NewSkills { get { return m_Skills; } set { m_Skills = value; } }
		public NewSkillNameValue ClassSkill_1 { get { return m_Skills[0]; } set { m_Skills[0] = value; } }
		public NewSkillNameValue ClassSkill_2 { get { return m_Skills[1]; } set { m_Skills[1] = value; } }
		public NewSkillNameValue ClassSkill_3 { get { return m_Skills[2]; } set { m_Skills[2] = value; } }
		public NewSkillNameValue ClassSkill_4 { get { return m_Skills[3]; } set { m_Skills[3] = value; } }
		public NewSkillNameValue ClassSkill_5 { get { return m_Skills[4]; } set { m_Skills[4] = value; } }

		[CommandProperty( AccessLevel.GameMaster )]
		public int BaseStrength { get { return m_Stats.Strength; } set { m_Stats.Strength = value; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int BaseDexterity { get { return m_Stats.Dexterity; } set { m_Stats.Dexterity = value; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int BaseIntelligence { get { return m_Stats.Intelligence; } set { m_Stats.Intelligence = value; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int BaseConstitution { get { return m_Stats.Constitution; } set { m_Stats.Constitution = value; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int BaseCharisma { get { return m_Stats.Charisma; } set { m_Stats.Charisma = value; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int BaseWisdom { get { return m_Stats.Wisdom; } set { m_Stats.Wisdom = value; } }

		public int ActualStrength { get { return BaseStrength + RaceEntry.GetEntry( _Race ).RaceMods[Stat.Strength]; } }
		public int ActualDexterity { get { return BaseDexterity + RaceEntry.GetEntry( _Race ).RaceMods[Stat.Dexterity]; } }
		public int ActualIntelligence { get { return BaseIntelligence + RaceEntry.GetEntry( _Race ).RaceMods[Stat.Intelligence]; } }
		public int ActualConstitution { get { return BaseConstitution + RaceEntry.GetEntry( _Race ).RaceMods[Stat.Constitution]; } }
		public int ActualCharisma { get { return BaseCharisma + RaceEntry.GetEntry( _Race ).RaceMods[Stat.Charisma]; } }
		public int ActualWisdom { get { return BaseWisdom + RaceEntry.GetEntry( _Race ).RaceMods[Stat.Wisdom]; } }

		public int GetActual( Stat name ) 
		{
			switch ( name )	
			{	
				case Stat.Strength: return ActualStrength;
				case Stat.Dexterity: return ActualDexterity;
				case Stat.Intelligence: return ActualIntelligence;
				case Stat.Constitution: return ActualConstitution;
				case Stat.Charisma: return ActualCharisma;
				case Stat.Wisdom: return ActualWisdom;
				default: return 0;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int StrengthBonus { get { return m_Stats.StrengthBonus; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int DexterityBonus { get { return m_Stats.DexterityBonus; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int IntelligenceBonus { get { return m_Stats.IntelligenceBonus; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int ConstitutionBonus { get { return m_Stats.ConstitutionBonus; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int CharismaBonus { get { return m_Stats.CharismaBonus; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int WisdomBonus { get { return m_Stats.WisdomBonus; } }

		[CommandProperty( AccessLevel.GameMaster )]
		public int StrengthCap { get { return m_Stats.StrengthCap; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int DexterityCap { get { return m_Stats.DexterityCap; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int IntelligenceCap { get { return m_Stats.IntelligenceCap; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int ConstitutionCap { get { return m_Stats.ConstitutionCap; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int CharismaCap { get { return m_Stats.CharismaCap; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int WisdomCap { get { return m_Stats.WisdomCap; } }

		
		[CommandProperty( AccessLevel.GameMaster )]
		public Race _Race { get { return m_Stats.Race; } set { m_Stats.Race = value; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public Class _Class { get { return m_Stats.Class; } set { m_Stats.Class = value; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int _Experience { get { return m_Stats.Experience; } set { m_Stats.Experience = value; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int _ClassLevel { get { return m_Level.CurrentLevel; } }
		[CommandProperty( AccessLevel.GameMaster )]
		public int _NextLevelPointsNeeded { get { return m_Level.PointsNeeded(); } }
		[CommandProperty( AccessLevel.GameMaster )]
		public string _ClassRank { get { return m_Level.GetRank(); } }
		
		public NewMobile()
		{
			m_Stats = new Stats( this, true );
			m_Skills = new NewSkillNameValue[5];
			m_Level = new LevelEntry( this );
		}
		
		public NewMobile( Serial s ) : base( s )
		{
		}

		public bool UseNewSkill( NewSkillName name )
		{
			return UseNewSkill( (int)name );
		}

		public bool UseNewSkill( int newSkillID )
		{
			NewSkillInfo info;
			if (!this.CheckAlive())
			{
				return false;
			}

			if (!NewSkillInfo.Table[newSkillID].ClickToUse)
			{
				this.SendMessage("You may not call this skill directly.");
				return false;
			}
			
			if ((newSkillID >= 0) && (newSkillID < NewSkillInfo.Table.Length))
			{
				info = NewSkillInfo.Table[newSkillID];
				if (this.Spell == null)
				{
					if (this.Meditating)
					{
						this.Meditating = false;
						this.SendLocalizedMessage(500134);
					}

					if ( info.Callback != null )
					{
						this.NextSkillTime = DateTime.Now + info.Callback( this );
						Console.WriteLine( this.NextSkillTime.ToString() );
					}
					
					this.SendMessage("Now using skill: {0}.", info.Name);
					return true;
				}
			}

			return false;
		}

		public bool CheckNewSkill( NewSkillName skill, double min, double max )
		{
			int num = (int)skill;
			if ( NewSkills[num%5].Name != skill )
				return false;
			double value = NewSkills[num%5].Value;
			if ( value < min )
				return false; // Too difficult
			else if ( value >= max )
				return true; // No challenge

			double chance = (value - min) / (max - min);

			return CheckNewSkill( skill, chance );
		}

		public bool CheckNewSkill( NewSkillName skill, double chance )
		{
			bool success = ( chance >= Utility.RandomDouble() );

			return success;
		}

		public override void GetContextMenuEntries( Mobile from, List<ContextMenuEntry> list )
		{
			base.GetContextMenuEntries( from, list );

			if ( from.Alive )
			{
				list.Add( new ContextMenus.NewSkillsEntry( from ) );
				list.Add( new ContextMenus.NewStatsEntry( from ) );
			}
		}
		
		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();

			switch ( version )
			{
				case 2:
				{
					m_Stats = new Stats( this, false );
					m_Stats.Deserialize( this, reader );

					m_Skills = new NewSkillNameValue[5];

					m_Level = new LevelEntry( this );
					m_Level.Deserialize( this, reader );
					goto case 1;
				}

				case 1:
				{
					for (int x=0; x<5; x++)
					{
						m_Skills[x].Name = (NewSkillName)reader.ReadInt();
						m_Skills[x].Value = (int)reader.ReadInt();
					}
					goto case 0;
				}
				case 0:
				{
					break;
				}
			}
		}
		
		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			
			writer.Write( (int) 2 ); // version

			m_Stats.Serialize( this, writer );

			m_Level.Serialize( this, writer );

			for (int x=0; x<5; x++)
			{
				writer.Write( (int) m_Skills[x].Name );
				writer.Write( (int) m_Skills[x].Value );
			}
		}
		
		public int Between( int number, int lower, int upper )
		{
			if ( number < lower ) return lower;
			else if ( number > upper ) return upper;
			else return number;
		}
		
		public int Lesser( int first, int second )
		{
			if ( first < second ) return first; else return second;
		}
		
		public int Greater( int first, int second )
		{
			if ( first > second ) return first; else return second;
		}
		
#region New additions to Override PlayerMobile defaults...
		
		public override int MaxWeight { get { return Between( ( ( this.ActualStrength + 5 ) * ( this.ActualStrength + 2 ) ), 40, 800 ); } }

		///TODO: find a better place to adjust the FollowersMax...
		public override void ValidateSkillMods()
		{
			try
			{
				int mod = (int)( ( this.ActualCharisma - 12 ) / 3 );
				this.FollowersMax = Between( 5 + mod, 2, 10 );
			}
			catch { }
			base.ValidateSkillMods();
		}

		///TODO: Add code to adjust resistances based on Class or Race abilities.
		public override int BasePhysicalResistance { get { return 0; } }
		public override int BaseFireResistance { get { return 0; } }
		public override int BaseColdResistance { get { return 0; } }
		public override int BasePoisonResistance { get { return 0; } }
		public override int BaseEnergyResistance { get { return 0; } }

		public override int GetMaxResistance( ResistanceType type )
		{
			int max = base.GetMaxResistance( type );

			return max;
		}

		public override int GetResistance( ResistanceType type )
		{
			return base.GetResistance( type );
		}

		public override int GetMinResistance( ResistanceType type )
		{
			return base.GetMinResistance( type );
		}

		public override void OnItemAdded( Item item )
		{
			base.OnItemAdded( item );
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public override int HitsMax
		{
			get
			{
				try {
					int baseHits = 50;
					int thisHits = baseHits + (int)( ( this.ActualStrength - 10 ) * 10 );
					return Greater( thisHits, 10 ); }
				catch{ return base.HitsMax; }
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public override int StamMax
		{
			get
			{
				try {
					int baseStam = 50;
					int thisStam = baseStam + (int)( ( this.ActualConstitution - 10 ) * 10 );
					return Greater( thisStam, 10 ); }
				catch { return base.StamMax; }
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public override int ManaMax
		{
			get
			{ 
				try {
					int baseMana = 50;
					int thisMana = baseMana + (int)( ( this.ActualIntelligence + this.ActualWisdom - 20 ) * 5 );
					return Greater( thisMana, 10 );  }
				catch { return base.ManaMax; }
			}
		}

		public override void Damage( int amount, Mobile from )
		{
			///TODO: Add code to determine how STR and certain skills affect Damage, like Warrior's DoubleDamage skill
			base.Damage( amount, from );
		}

		public override void GetProperties( ObjectPropertyList list )
		{
			///TODO: Add code to display Race or Class titles
			base.GetProperties( list );
		}

		public override void OnSingleClick( Mobile from )
		{
			///TODO: Add code to display Race or Class titles
			base.OnSingleClick( from );
		}

		public override string ApplyNameSuffix( string suffix )
		{
			///TODO: Add code to display Race or Class titles
			return base.ApplyNameSuffix( suffix );
		}
		
#endregion
// End of New additions to Override PlayerMobile defaults...
	}
}