using System;
using Server.Targeting;
using Server.Items;
using Server.Network;
using Server.Multis;
using Server.Mobiles;
using Server.Custom;

namespace Server.SkillHandlers
{
	public class Hypnotism
	{
		public static void Initialize()
		{
			NewSkillInfo.Table[(int)NewSkillName.Hypnotism].Callback = new SkillUseCallback( OnUse );
		}

		public static TimeSpan OnUse( Mobile m )
		{
			m.Target = new InternalTarget();
			m.SendMessage("Who do you wish to hypnotize?");
			return TimeSpan.FromSeconds( 9.0 );
		}

		private class InternalTarget : Target
		{
			private bool m_SetSkillTime = true;

			public InternalTarget() :  base ( 12, false, TargetFlags.None )
			{
			}

			protected override void OnTargetFinish( Mobile from )
			{
				if ( m_SetSkillTime )
					from.NextSkillTime = DateTime.Now;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				from.RevealingAction();

				if ( targeted is Mobile )
				{
					if ( targeted is NewMobile )
					{
						NewMobile mobile = targeted as NewMobile;
						from.Direction = from.GetDirectionTo( mobile );

						from.LocalOverheadMessage( MessageType.Emote, 0x59, true, String.Format( "You begin to hypnotize {0}", mobile.Name ) );
						from.NonlocalOverheadMessage( MessageType.Emote, 0x59, true, String.Format( "* begins hypnotizing {0} *", mobile.Name ) );

						new InternalTimer( from, mobile ).Start();

						m_SetSkillTime = false;
					}
					else if ( targeted is BaseCreature )
					{
						BaseCreature creature = targeted as BaseCreature;
						from.Direction = from.GetDirectionTo( creature );

						from.LocalOverheadMessage( MessageType.Emote, 0x59, true, String.Format( "You begin to hypnotize {0}", creature.Name ) );
						from.NonlocalOverheadMessage( MessageType.Emote, 0x59, true, String.Format( "* begins hypnotizing {0} *", creature.Name ) );

						new InternalTimer( from, creature ).Start();

						m_SetSkillTime = false;
					}
					else
					{
						from.SendMessage("That being will not be hypnotized.");
					}
				}
				else
				{
					from.SendMessage("You can not hypnotize that!");
				}

			}

			private class InternalTimer : Timer
			{
				private Mobile m_From;
				private NewMobile m_MobileTarget;
				private BaseCreature m_CreatureTarget;
				private bool isCreature;

				public InternalTimer( Mobile from, NewMobile mobiletarget ) : base( TimeSpan.FromSeconds( 2.0 ) )
				{
					m_From = from;
					m_MobileTarget = mobiletarget;
					Priority = TimerPriority.TwoFiftyMS;
					isCreature = false;
				}

				public InternalTimer( Mobile from, BaseCreature creaturetarget ) : base( TimeSpan.FromSeconds( 2.0 ) )
				{
					m_From = from;
					m_CreatureTarget = creaturetarget;
					Priority = TimerPriority.TwoFiftyMS;
					isCreature = true;
				}

				protected override void OnTick()
				{
					if ( isCreature )
					{
						m_From.SendMessage("At this point you will see a Hypnotize Creature Gump.");
					}
					else
					{
						m_From.SendMessage("At this point you will see a Hypnotize Player Gump.");
					}

				}
			}
		}
	}
}