using Server;
using System;
using System.Collections;

namespace Server.Custom
{
	
	public enum Race
	{
		None = -1,
		
		Human = 0,
		Elf,
		Dwarf,
		HalfElf,
		Hobbit,
		Gnome,
		HalfOrc
		
	}

	public class RaceEntry
	{
		private Race m_Race;
		private string m_Name;
		private string m_Desc;
		private Stats m_Mods;

		public Race Race{ get{ return m_Race; } }
		public string Name{ get{ return m_Name; } }
		public string Description{ get{ return m_Desc; } }
		public Stats RaceMods{ get{ return m_Mods; } }
		 
		private Stats m_Owner;
		public Stats Owner{get{ return m_Owner; }set{ m_Owner = value; }}

		public RaceEntry( Race race, string name, string description, Stats raceMods )
		{		
			m_Race = race;
			m_Name = name;
			m_Desc = description;
			m_Mods = raceMods;
		}

		public RaceEntry( Stats owner )
		{
			m_Owner = owner;
		}

		private static Hashtable m_Table = new Hashtable();
		public static Hashtable Table{ get{ return m_Table; } }
		
		public static RaceEntry GetEntry( Race race )
		{
			return ( m_Table[race] as RaceEntry );
		}
		
		public static void Register( RaceEntry raceEntry )
		{
			if ( !m_Table.ContainsKey( raceEntry.Race ) )
				m_Table[raceEntry.Race] = raceEntry;
			else
				throw new Exception( String.Format( "Race {0} exists.", raceEntry.Race.ToString() ) );
		}
	}

	public sealed class NoneRace
	{
		public static void Initialize()
		{
			Race race = Race.None;
			string name = "None";
			string description = "No Race Chosen.";
			Stats stats = new Stats( race, 0, 0, 0, 0, 0, 0 );
			RaceEntry.Register( new RaceEntry( race, name, description, stats ) );
		}
	}

	public sealed class Human
	{
		public static void Initialize()
		{
			Race race = Race.Human;
			string name = "Human";
			string description = "Their strength and inventiveness make them one of the best survivors.  Never lacking in aggression, they can handle themselves in just about any adverse setting.";
			Stats stats = new Stats( race, 4, 2, 2, 2, 2, 4 );
			RaceEntry.Register( new RaceEntry( race, name, description, stats ) );
		}
	}

	public sealed class Elf
	{
		public static void Initialize()
		{
			Race race = Race.Elf;
			string name = "Elf";
			string description = "Fair of hair and skin, the Elves are slightly taller, faster, and more agile than humans.  Although weaker, they make up for this by being more skilled in magical arts and war strategies.";
			Stats stats = new Stats( race, 0, 5, 5, 0, 4, 2 );
			RaceEntry.Register( new RaceEntry( race, name, description, stats ) );
		}
	}

	public sealed class Dwarf
	{
		public static void Initialize()
		{
			Race race = Race.Dwarf;
			string name = "Dwarf";
			string description = "Stout and short, these people are amazingly strong and healthy.  Brightness, however, is not a shining attribute of theirs.";
			Stats stats = new Stats( race, 6, 2, 0, 6, 2, 0 );
			RaceEntry.Register( new RaceEntry( race, name, description, stats ) );
		}
	}

	public sealed class HalfElf
	{
		public static void Initialize()
		{
			Race race = Race.HalfElf;
			string name = "Half-Elf";
			string description = "The result of an Elf and Human friendship, Half-Elves acquire their fair hair and light skin from Elves, and their physical bulk from Humans.";
			Stats stats = new Stats( race, 3, 4, 4, 2, 3, 0 );
			RaceEntry.Register( new RaceEntry( race, name, description, stats ) );
		}
	}

	public sealed class Hobbit
	{
		public static void Initialize()
		{
			Race race = Race.Hobbit;
			string name = "Hobbit";
			string description = "The Hobbit is deft and clever, which are ideal traits for thieving.  Their favorite boast is -- A locked door is soon no more.";
			Stats stats = new Stats( race, 2, 6, 4, 2, 2, 0 );
			RaceEntry.Register( new RaceEntry( race, name, description, stats ) );
		}
	}

	public sealed class Gnome
	{
		public static void Initialize()
		{
			Race race = Race.Gnome;
			string name = "Gnome";
			string description = "Gnomes are much like Dwarfs, but a little more anti-social due to a deficit in beauty.  Because they've spent so much time studying alone, Gnomes have developed a certain flair for magic.";
			Stats stats = new Stats( race, 5, 1, 3, 4, 0, 3 );
			RaceEntry.Register( new RaceEntry( race, name, description, stats ) );
		}
	}

	public sealed class HalfOrc
	{
		public static void Initialize()
		{
			Race race = Race.HalfOrc;
			string name = "Half-Orc";
			string description = "Orcs are the henchmen of evil wizards who are capable of little more than following orders for physical dirty work.  Half-Orcs are tempered with Human blood, so while they're still dangerous, they aren't completely mindless drones forever in search of necks to wrench.";
			Stats stats = new Stats( race, 6, 4, 2, 4, 0, 0 );
			RaceEntry.Register( new RaceEntry( race, name, description, stats ) );
		}
	}


}