using System;
using System.Collections;
using Server;
using Server.Items;
using Server.Mobiles;
using Server.Gumps;

namespace Server.Custom
{
	public enum Stat
	{
		Strength,
		Dexterity,
		Intelligence,
		Constitution,
		Charisma,
		Wisdom
	}

	public class Stats
	{
		private const int DC = 16; // Default Cap
		private const int DV = 6; // Default Value
		private NewMobile m_Owner;
		private Race m_Race;
		private Class m_Class;
		private int m_Exp;
		private int m_Str, m_Dex, m_Int, m_Con, m_Cha, m_Wis;
		public NewMobile Owner{ get { return m_Owner; } set { m_Owner = value; } }
		public Race Race { get { return m_Race; } set { m_Race = value; } }
		public Class Class { get { return m_Class; } set { m_Class = value; } }
		public int Experience { get { return m_Exp; } set { m_Exp = value; } }
		public int Strength { get { return m_Str; } set { m_Str = value; } }
		public int Dexterity { get { return m_Dex; } set { m_Dex = value; } }
		public int Intelligence { get { return m_Int; } set { m_Int = value; } }
		public int Constitution { get { return m_Con; } set { m_Con = value; } }
		public int Charisma { get { return m_Cha; } set { m_Cha = value; } }
		public int Wisdom { get { return m_Wis; } set { m_Wis = value; } }

		public int StrengthBonus{ get{ return CalculateBonus( m_Owner.ActualStrength ); } }
		public int DexterityBonus{ get{ return CalculateBonus( m_Owner.ActualDexterity ); } }
		public int IntelligenceBonus{ get{ return CalculateBonus( m_Owner.ActualIntelligence ); } }
		public int ConstitutionBonus{ get{ return CalculateBonus( m_Owner.ActualConstitution ); } }
		public int CharismaBonus{ get{ return CalculateBonus( m_Owner.ActualCharisma ); } }
		public int WisdomBonus{ get{ return CalculateBonus( m_Owner.ActualWisdom ); } }
		
		public int StrengthCap{ get{ return CalculateCap( Stat.Strength ); } }
		public int DexterityCap{ get{ return CalculateCap( Stat.Dexterity ); } }
		public int IntelligenceCap{ get{ return CalculateCap( Stat.Intelligence ); } }
		public int ConstitutionCap{ get{ return CalculateCap( Stat.Constitution ); } }
		public int CharismaCap{ get{ return CalculateCap( Stat.Charisma ); } }
		public int WisdomCap{ get{ return CalculateCap( Stat.Wisdom ); } }

		public static int CalculateBonus( int score )
		{
			return (int)Math.Floor( ( score - 10 ) / 2.0 );
		}

		public int CalculateCap( Stat name )
		{
			if ( m_Race != Race.None ) {
				RaceEntry race = RaceEntry.GetEntry( m_Race );
				return ( DC + race.RaceMods[name] ); }
			else {
				return DC; }
		}

		public int this[ int index ] {
			get{ return this[(Stat)index]; }
			set{ this[(Stat)index] = value; } }

		public int this[ Stat name ] {
			get {
				switch ( name )	{	
					case Stat.Strength: return m_Str;
					case Stat.Dexterity: return m_Dex;
					case Stat.Intelligence: return m_Int;
					case Stat.Constitution: return m_Con;
					case Stat.Charisma: return m_Cha;
					case Stat.Wisdom: return m_Wis;
					default: return 0; } }
			set {
				switch ( name ) {
					case Stat.Strength: Strength = value; break;
					case Stat.Dexterity: Dexterity = value; break;
					case Stat.Intelligence: Intelligence = value; break;
					case Stat.Constitution: Constitution = value; break;
					case Stat.Charisma: Charisma = value; break;
					case Stat.Wisdom: Wisdom = value; break; } }
		}
		
		public Stats( Race race, int Str, int Dex, int Int, int Con, int Cha, int Wis )
			: this( null, race, Class.None, 0, Str, Dex, Int, Con, Cha, Wis )
		{
		}

		public Stats( NewMobile owner, bool isNew ) : this( owner, Race.None, Class.None, 0, DV, DV, DV, DV, DV, DV )
		{
		}

		public Stats( NewMobile owner ) : this( owner, owner._Race, owner._Class, owner._Experience,
			owner.BaseStrength, owner.BaseDexterity, owner.BaseIntelligence, owner.BaseConstitution, owner.BaseCharisma, owner.BaseWisdom )
		{
		}
		
		public Stats( NewMobile owner, Race Rac, Class Cla, int Exp, int Str, int Dex, int Int, int Con, int Cha, int Wis )
		{
			m_Owner = owner;
			m_Race = Rac;
			m_Class = Cla;
			m_Exp = Exp;
			m_Str = Str;
			m_Dex = Dex;
			m_Int = Int;
			m_Con = Con;
			m_Cha = Cha;
			m_Wis = Wis;
		}

		public void Deserialize( NewMobile from, GenericReader reader )
		{
			int version = reader.ReadInt();

			switch ( version )
			{
				case 2:
				{
					m_Owner = from;
					m_Race = (Server.Custom.Race)reader.ReadInt();
					m_Class = (Server.Custom.Class)reader.ReadInt();
					m_Exp = reader.ReadInt();
					goto case 1;
				}

				case 1:
				{
					m_Str = reader.ReadInt();
					m_Dex = reader.ReadInt();
					m_Int = reader.ReadInt();
					m_Con = reader.ReadInt();
					m_Cha = reader.ReadInt();
					m_Wis = reader.ReadInt();
					goto case 0;
				}
				case 0: break;
			}
		}
		
		public void Serialize( NewMobile from, GenericWriter writer )
		{
			writer.Write( (int) 2 ); // version

			m_Owner = from;
			writer.Write( (int) m_Race );
			writer.Write( (int) m_Class );
			writer.Write( (int) m_Exp );

			writer.Write( (int) m_Str );
			writer.Write( (int) m_Dex );
			writer.Write( (int) m_Int );
			writer.Write( (int) m_Con );
			writer.Write( (int) m_Cha );
			writer.Write( (int) m_Wis );
		}
	}
}